//---------------------------------------------------------------------------
#ifndef DecimalH
#define DecimalH

#define MAX_PRECISION       4
#define DEFAULT_PRECISION   2
//---------------------------------------------------------------------------

class PACKAGE Decimal
{
    unsigned Precision;
    friend __int64 __fastcall __RoundInt64(__int64 value,const unsigned precision);
    friend Decimal __fastcall Round(Decimal value,const unsigned precision);
    friend Decimal __fastcall operator +(int lhs, const Decimal& rhs);
    friend Decimal __fastcall operator -(int lhs, const Decimal& rhs);
    friend Decimal __fastcall operator *(int lhs, const Decimal& rhs);
    friend Decimal __fastcall operator /(int lhs, const Decimal& rhs);

    friend Decimal __fastcall operator +(double lhs, const Decimal& rhs);
    friend Decimal __fastcall operator -(double lhs, const Decimal& rhs);
    friend Decimal __fastcall operator *(double lhs, const Decimal& rhs);
    friend Decimal __fastcall operator /(double lhs, const Decimal& rhs);
public:
    __int64 Val;
    __fastcall Decimal();
    __fastcall Decimal(const Decimal& src);
    __fastcall Decimal(const Decimal& src, unsigned int precision);
    __fastcall Decimal(const __int64 val,unsigned int precision=DEFAULT_PRECISION);
    __fastcall Decimal(const double val,unsigned int precision=DEFAULT_PRECISION);
    __fastcall Decimal(const int val,unsigned int precision=DEFAULT_PRECISION);
    __fastcall Decimal(const AnsiString& src,unsigned int precision=DEFAULT_PRECISION);

    void __fastcall SetPrecision(const unsigned precision);

    Decimal& __fastcall operator =(double rhs);
    Decimal& __fastcall operator =(int rhs);
    Decimal& __fastcall operator =(const Decimal& rhs);

    Decimal& __fastcall operator +=(const Decimal& rhs);
    Decimal& __fastcall operator -=(const Decimal& rhs);
    Decimal& __fastcall operator *=(const Decimal& rhs);
    Decimal& __fastcall operator /=(const Decimal& rhs);
    Decimal& __fastcall operator %=(int rhs);

    Decimal& operator ++();
    Decimal operator ++(int);
    Decimal& operator --();
    Decimal operator --(int);

    Decimal __fastcall operator +(const Decimal& rhs) const;
    Decimal __fastcall operator -(const Decimal& rhs) const;
    Decimal __fastcall operator *(const Decimal& rhs) const;
    Decimal __fastcall operator /(const Decimal& rhs) const;

    Decimal __fastcall operator +(int rhs) const;
    Decimal __fastcall operator -(int rhs) const;
    Decimal __fastcall operator *(int rhs) const;
    Decimal __fastcall operator /(int rhs) const;
    Decimal __fastcall operator %(int rhs) const;

    Decimal __fastcall operator +(double rhs) const;
    Decimal __fastcall operator -(double rhs) const;
    Decimal __fastcall operator *(double rhs) const;
    Decimal __fastcall operator /(double rhs) const;

    Decimal __fastcall operator -() const;

    Decimal __fastcall operator !() const;

    // comparisons (Decimal rhs)
    bool __fastcall operator ==(const Decimal& rhs) const;
    bool __fastcall operator !=(const Decimal& rhs) const;
    bool __fastcall operator >(const Decimal& rhs) const;
    bool __fastcall operator <(const Decimal& rhs) const;
    bool __fastcall operator >=(const Decimal& rhs) const;
    bool __fastcall operator <=(const Decimal& rhs) const;

    // comparisons (int rhs)
    bool __fastcall operator ==(const int rhs) const;
    bool __fastcall operator !=(const int rhs) const;
    bool __fastcall operator >(const int rhs) const;
    bool __fastcall operator <(const int rhs) const;
    bool __fastcall operator >=(const int rhs) const;
    bool __fastcall operator <=(const int rhs) const;

    // comparisons (double rhs)
    bool __fastcall operator ==(double rhs) const;
    bool __fastcall operator !=(double rhs) const;
    bool __fastcall operator >(double rhs) const;
    bool __fastcall operator <(double rhs) const;
    bool __fastcall operator >=(double rhs) const;
    bool __fastcall operator <=(double rhs) const;

    __fastcall operator double() const;
    __fastcall operator int() const;
    __fastcall operator AnsiString() const;
};

inline Decimal __fastcall operator +(int lhs, const Decimal& rhs);
inline Decimal __fastcall operator -(int lhs, const Decimal& rhs);
inline Decimal __fastcall operator *(int lhs, const Decimal& rhs);
inline Decimal __fastcall operator /(int lhs, const Decimal& rhs);

inline Decimal __fastcall operator +(double lhs, const Decimal& rhs);
inline Decimal __fastcall operator -(double lhs, const Decimal& rhs);
inline Decimal __fastcall operator *(double lhs, const Decimal& rhs);
inline Decimal __fastcall operator /(double lhs, const Decimal& rhs);
inline __int64 __fastcall __RoundInt64(__int64 value,const unsigned precision);
Decimal __fastcall Round(Decimal value,const unsigned precision);

//---------------------------------------------------------------------------
#endif
